//---------------------------------------------------------------------------------------
/// \file	CircleSelectTool.h
/// \author	Rob Bateman
/// \brief	Right, well having a rectangular selection tool is pretty useless. This example
/// 		will add in a custom selection tool that will actually select things in a circle,
/// 		rather than a rectangle.
///
/// 			This process involves a little bit of a hack, basically we :
///
///				* select the items using a rectangular area of the screen.
/// 			* Then query the current selection list via MGlobal
///				* iterate through each component
///					* transform it into 2D coords
/// 				* add any components whose 2D coord falls within the circle to a selection list
/// 			* select the new selection list
///
/// 			Unfortunately, there are a lot of component types that we really need to
/// 		iterate over. To start with then, this example will only iterate over mesh
/// 		vertices and nothing else.
///
/// 			The Tool3 example is a somewhat improved version of the lassoTool that comes
/// 		in the maya API. The only difference is that mine works properly and supports
/// 		all of the component types.
///
/// 			C++ templates might be worth looking at for the component iterating, it
/// 		is likely so save you some time in the long run ;)
///
//---------------------------------------------------------------------------------------

#ifndef __CIRCLE_SELECT_TOOL__H__
#define __CIRCLE_SELECT_TOOL__H__

	#ifdef WIN32
		#define NT_PLUGIN
	#endif

	#include <maya/MIOStream.h>
	#include <math.h>
	#include <stdlib.h>

	#include <maya/MString.h>
	#include <maya/MGlobal.h>
	#include <maya/M3dView.h>
	#include <maya/MDagPath.h>

	#include <maya/MItSelectionList.h>

	#include <maya/MPxContextCommand.h>
	#include <maya/MPxContext.h>
	#include <maya/MCursor.h>
	#include <maya/MEvent.h>

	#include <GL/gl.h>
	#include <GL/glu.h>

	#include <maya/MPoint.h>
	#include <maya/MPointArray.h>
	#include <maya/MItGeometry.h>
	#include <maya/MItCurveCV.h>
	#include <maya/MItSurfaceCV.h>
	#include <maya/MItMeshVertex.h>
	#include <maya/MItMeshEdge.h>
	#include <maya/MItMeshPolygon.h>


	//-----------------------------------------------------------------------------------	CircleSelectTool
	///	\brief	The main part of a tool plugin is the Context part. This is
	/// 		the class that handles the mouse interaction from the user
	/// 		for your tool. In this fairly simple example, the class simply
	/// 		stores and updates a start and end point between which a
	/// 		rectangle will be drawn.
	///
	class CircleSelectTool
		: public MPxContext
	{
	public:

		/// \brief	ctor
		CircleSelectTool();

		/// \brief	dtor
		virtual ~CircleSelectTool();

		/// \brief	This is used to setup any user interface things you need when the tool
		/// 		is made current.
		/// \param	event	-	information on the input event
		///
		virtual void	toolOnSetup( MEvent & event );

		/// \brief	overide to handle a mouse press event
		/// \param	event	-	information on the input event
		/// \return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus	doPress( MEvent & event );

		/// \brief	overide to handle a mouse drag event
		/// \param	event	-	information on the input event
		/// \return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus	doDrag( MEvent & event );

		/// \brief	overide to handle a mouse release event
		/// \param	event	-	information on the input event
		/// \return	MS::kSuccess or MS::kFailure
		///
		virtual MStatus	doRelease( MEvent & event );

	private:

		/// \brief	 an internal utility function to determine if the specified 2D point is
		///		 	within the selection circle
		/// \param	x	-	the x coord of the point to set
		/// \param	y	-	the y coord of the point to set
		/// \return	true if OK
		///
		bool ShouldAddPoint(const MPoint& p3d);

		/// The location of the cursor when the mouse was first clicked
		short m_Start[2];

		/// The location of the cursor when the mouse gets released
		short m_End[2];

		/// The radius of the circle, calculated from m_Start and m_End
		short m_Radius;

		/// The View we are doing the selection in.
		M3dView m_View;

	};

	//-----------------------------------------------------------------------------------	CircleSelectToolCommand
	/// \brief	This class represents a simple mel command that will allow
	/// 		the user to enter a context for our tool. It simply has to return
	/// 		an instance of the Sontext class.
	///
	class CircleSelectToolCommand
		: public MPxContextCommand
	{
	public:

		/// \brief	This function needs to return a context for maya to use
		/// 		when our tool requires input from the viewport.
		/// \return	a new instance of CircleSelectTool
		///
		virtual MPxContext* makeObj();

		/// \brief	This returns a new instance of the tool command for maya to use.
		/// \return	an instance of RectangleSelectContextCmd
		///
		static void*		creator();
	};


#endif
